/**************************************************************************
// PICO_SIGNATURE
* 
* Description:
*   This is a console-mode program that demonstrates how to call the
*   USBPT104 driver API functions using C#.
*
***************************************************************************/

using System;
using System.IO;
using System.Text;
using System.Threading;

using USBPT104Imports;

namespace USBPT104CSConsole
{
    class ConsoleExample
    {
        private readonly short _handle;
        public const uint USBPT104_MAX_CHANNELS = 4;
       
        private static void WaitForKey()
        {
            while(!Console.KeyAvailable)
            {
                Thread.Sleep(100);
            }

            if (Console.KeyAvailable)
            {
                Console.ReadKey(true); // clear the key
            }
        }

        /****************************************************************************
         * Read the device information
         ****************************************************************************/
        void GetDeviceInfo()
        {
            short required_size = 0;
            StringBuilder line = new System.Text.StringBuilder(256);
            short lineLength = (short) line.Capacity;

            if (_handle >= 0)
            {
                Console.WriteLine("Device Info:\n");
                
                Imports.UsbPt104GetUnitInfo(_handle, line, lineLength, ref required_size, Imports.PICO_BATCH_AND_SERIAL);
                Console.WriteLine("Batch and serial: {0}", line);
                
                Imports.UsbPt104GetUnitInfo(_handle, line, lineLength, ref required_size, Imports.PICO_DRIVER_VERSION);
                Console.WriteLine("Driver version: {0}", line);
                
                Imports.UsbPt104GetUnitInfo(_handle, line, lineLength, ref required_size, Imports.PICO_CAL_DATE);
                Console.WriteLine("Calibration date: {0}", line);
            }
            else
            {
                Console.WriteLine("Error: Failed to get Device Info\n");
            }
        }

   
        /****************************************************************************
         * Read temperature information from the unit
         ****************************************************************************/
        unsafe void GetValues()
        {
            uint status = Imports.PICO_OK;
            uint chan;
            int []tempbuffer = new int[4]; // Single ended measurements; use 8 for differential and update for loop below

            Console.Write("\n");
            Console.WriteLine("Press any key to stop....\n");
            Console.Write("\n");

            // Label the columns
            for (chan = 0; chan < USBPT104_MAX_CHANNELS; chan++)
            {   
                Console.Write("Chan{0}:\t\t", chan+1);
            }
            Console.Write("\n");

            do
            {
                    for (chan = 0; chan < USBPT104_MAX_CHANNELS; chan++)
                    {
                        status = Imports.UsbPt104GetValue( _handle, (Imports.UsbPt104Channels) (chan + 1), out tempbuffer[chan], 0);

                        if (status == Imports.PICO_OK)
                        {
                            // Divide by 1000 for PT100 and PT1000 sensors         
                            Console.Write("{0:000.0000}\t", tempbuffer[chan] / 1000.0000);
                        }
                        else
                        {
                            Console.Write("{0:000.0000}\t", 0);
                        }
                             
                        Thread.Sleep(720); //Delay for min. hardware channel reading time, to get new reading

                    }
                    Console.Write("\n");

            } while (!Console.KeyAvailable);

            char ch = (Console.ReadKey().KeyChar);       // use up keypress
            status = Imports.UsbPt104CloseUnit(_handle);
        }

        /****************************************************************************
        *  Set channels 
        ****************************************************************************/
        
        void SetChannels()
        {
	        uint channel = 0;
	        uint status = Imports.PICO_OK;

            Console.WriteLine("\nSetup all 4 channels for PT100 sensor with 4 wire connection.\n");
            for (channel = 0; channel < USBPT104_MAX_CHANNELS; channel++)
	        {
                // Setup all 4 channels for PT100 sensor with 4 wire connection
                status = Imports.UsbPt104SetChannel(_handle, (Imports.UsbPt104Channels) (channel + 1), Imports.UsbPt104DataType.USBPT104_PT100, Imports.USBPT104_MAX_WIRES);
            
	        }
            
        }

        /****************************************************************************
        *  Run
        ****************************************************************************/
        
        public void Run()
        {
             //// main loop - read key and call routine
            char ch = ' ';
            while (ch != 'X')
            {
                Console.WriteLine("");
                Console.WriteLine("I - Device Info");
                Console.WriteLine("S - Set Channels");
                Console.WriteLine("G - Get Temperatures");
                //Console.WriteLine("N - Setup IP network");
                //Console.WriteLine("P - Get Temperatures via IP network");
                Console.WriteLine("X - Exit");
                Console.WriteLine("\nOperation:");

                ch = char.ToUpper(Console.ReadKey().KeyChar);

                Console.WriteLine("\n");
                switch (ch)
                {
                    case 'I':
                        GetDeviceInfo();
                        break;

                    case 'S':
                        SetChannels();
                        break;

                    case 'G':
                        GetValues();
                        break;

                    case 'X':
                        // Handled by outer loop 
                        break;

                    default:
                        Console.WriteLine("Invalid operation");
                        break;
                }
            }
        }


        private ConsoleExample(short handle)
        {
            _handle = handle;
        }


        static void Main()
        {
            short handle;
            uint status = Imports.PICO_OK;

            StringBuilder detailsStr = new StringBuilder(256);
            uint enumerateLength = (uint) detailsStr.Capacity;

            Console.WriteLine("USB PT104 driver example program");
            Console.WriteLine("Version 1.2\n");

            Console.WriteLine("Enumerating devices...\n");

            // Enumerate all USB and Ethernet devices
            status = Imports.UsbPt104Enumerate(detailsStr, ref enumerateLength, Imports.CommunicationType.CT_ALL);

            if (status == Imports.PICO_OK)
            {
                Console.WriteLine("USB PT-104 devices found: {0}\n", detailsStr);
            }
            else
            {
                Console.WriteLine("No USB PT-104 devices found.");
            }

            Console.WriteLine("\nOpening the device...");

            status = Imports.UsbPt104OpenUnit(out handle, null);
            Console.WriteLine("Handle: {0}", handle);
      
            if (handle == 0)
            {
                Console.WriteLine("Unable to open device");
                Console.WriteLine("Error code : {0}", handle);
                WaitForKey();
            }
            else
            {
                Console.WriteLine("Device opened successfully\n");

                //WaitForKey();
                ConsoleExample consoleExample = new ConsoleExample(handle);
                consoleExample.Run();

                Imports.UsbPt104CloseUnit(handle);
            }
        }
    }
}  
